@@ -1,7 +1,7 @@
 /* $Header$ */
 
 /*
- * Copyright  1988-2007 Keith Packard and Bart Massey.
+ * Copyright © 1988-2007 Keith Packard and Bart Massey.
  * All Rights Reserved.  See the file COPYING in this directory
  * for licensing information.
  */
@@ -182,10 +182,18 @@ typedef int HashValue;
 typedef struct _natural {
     DataType	*type;
     int		length;
+    digit	digits[0];
 } Natural;
 
-#define NaturalLength(n)    ((n)->length)
-#define NaturalDigits(n)    ((digit *) ((n) + 1))
+static inline int NaturalLength(Natural *n)
+{
+	return n->length;
+}
+
+static inline digit * NaturalDigits(Natural *n)
+{
+	return n->digits;
+}
 
 Natural	*NewNatural (unsigned value);
 Natural *NewDoubleDigitNatural (double_digit dd);
@@ -233,19 +241,19 @@ extern Natural	*zero_natural;
 extern Natural	*one_natural;
 extern Natural	*two_natural;
 
-typedef enum _sign { Positive, Negative } Sign;
+typedef enum _sign { Positive = 0, Negative = 1 } Sign;
 
-#define SignNegate(sign)    ((sign) == Positive ? Negative : Positive)
+static inline Sign SignNegate(Sign sign) {
+	return 1 - sign;
+}
 
 typedef enum _signcat {
-	BothPositive, FirstPositive, SecondPositive, BothNegative
+	BothPositive = 0, FirstPositive = 1, SecondPositive = 2, BothNegative = 3
 } Signcat;
 
-# define catagorize_signs(s1,s2)\
-	((s1) == Positive ? \
-		((s2) == Positive ? BothPositive : FirstPositive) \
-	: \
-		((s2) == Positive ? SecondPositive : BothNegative))
+static inline Signcat catagorize_signs(Sign s1, Sign s2) {
+	return (s1 << 1) | s2;
+}
 
 typedef enum _binaryOp {
     PlusOp, MinusOp, TimesOp, DivideOp, DivOp, ModOp,
@@ -295,6 +303,8 @@ typedef enum _rep {
 /* because rep_undef is -1, using (unsigned) makes these a single compare */
 #define Numericp(t)	((unsigned) (t) <= (unsigned) rep_float)
 #define Integralp(t)	((unsigned) (t) <= (unsigned) rep_integer)
+
+
 #define Mutablep(t)	((t) >= rep_array)
 
 extern ValueRep    IntRep, IntegerRep, RationalRep, FloatRep;
@@ -304,8 +314,13 @@ extern ValueRep	   FuncRep, ThreadRep;
 extern ValueRep    SemaphoreRep, ContinuationRep, UnitRep, BoolRep;
 extern ValueRep    ForeignRep;
 
-#define NewInt(i)	((Value) IntToPtr ((((i) << 1) | 1)))
-#define IntSign(i)	((i) < 0 ? Negative : Positive)
+static inline Value NewInt(int i) {
+	return (Value) IntToPtr ((((i) << 1) | 1));
+}
+
+static inline Sign IntSign(int i) {
+	return (i) < 0 ? Negative : Positive;
+}
 
 /*
  * Use all but one bit to hold immediate integer values
@@ -336,11 +351,21 @@ extern ValueRep    ForeignRep;
 #define One NewInt(1)
 #define Zero NewInt(0)
 
-#define ValueIsPtr(v)	((PtrToInt(v) & 1) == 0)
-#define ValueIsInt(v)	(!ValueIsPtr(v))
-#define ValueInt(v)	(PtrToInt (v) >> 1)
+static inline Bool ValueIsPtr (Value v) {
+	return (PtrToInt(v) & 1) == 0;
+}
+	
+static inline Bool ValueIsInt (Value v) {
+	return !ValueIsPtr(v);
+}
 
-#define ValueRep(v) (ValueIsInt(v) ? &IntRep : (v)->value.type)
+static inline int ValueInt(Value v) {
+	return PtrToInt (v) >> 1;
+}
+
+static inline ValueRep *_ValueRep(Value v);
+
+#define ValueRep(v) _ValueRep(v)
 #define ValueIsInteger(v) (ValueRep(v) == &IntegerRep)
 #define ValueIsRational(v) (ValueRep(v) == &RationalRep)
 #define ValueIsFloat(v) (ValueRep(v) == &FloatRep)
@@ -395,8 +420,6 @@ typedef struct _typeName {
     SymbolPtr	name;
 } TypeName;
 
-#define TypeNameType(t)	((t)->name.name ? (t)->name.name->symbol.type : 0)
-
 typedef struct _typeRef {
     TypeBase	base;
     TypePtr	ref;
@@ -499,8 +522,6 @@ Bool	TypeTypesMember (Type *list, Type *type);
 int	TypeInit (void);
 SymbolPtr   TypeNameName (Type *t);
 
-#define TypeUnionElements(t) ((Type **) (&t->unions + 1))
-
 Type	*TypeCombineBinary (Type *left, int tag, Type *right);
 Type	*TypeCombineUnary (Type *down, int tag);
 Type	*TypeCombineStruct (Type *type, int tag, Atom atom);
@@ -542,7 +563,7 @@ typedef enum _publish {
     publish_private, publish_protected, publish_public, publish_extend
 } Publish;
 
-#define ValueTag(v) (ValueRep(v)->tag)
+static inline Rep ValueTag(Value v);
 
 typedef struct _baseValue {
     ValueRep	*type;
@@ -553,9 +574,6 @@ typedef struct _integer {
     Natural	*magn;
 } Integer;
 
-#define IntegerMag(i)	((Natural *) ((long) ((i)->integer.magn) & ~1))
-#define IntegerSign(i)	((Sign) ((long) ((i)->integer.magn) & 1))
-
 typedef struct _rational {
     BaseValue	base;
     Sign	sign;
@@ -579,9 +597,14 @@ typedef struct _float {
 typedef struct _string {
     BaseValue	    base;
     long	    length;
+    char	    chars[0];
 } String;
 
-#define StringChars(s)	    ((char *) ((s) + 1))
+static inline char *
+StringChars (String *s)
+{
+	return s->chars;
+}
 
 typedef struct _foreign {
     BaseValue	    base;
@@ -601,9 +624,13 @@ typedef struct _boxVector {
     DataType	*data;
     int		nvalues;
     TypePtr	type;
+    BoxPtr	boxes[0];
 } BoxVector, *BoxVectorPtr;
 
-#define BoxVectorBoxes(v)   ((BoxPtr *) ((v) + 1))
+static inline BoxPtr *BoxVectorBoxes(BoxVector *v)
+{
+	return (BoxPtr *) v->boxes;
+}
 
 typedef struct _array {
     BaseValue	base;
@@ -613,24 +640,15 @@ typedef struct _array {
 	BoxPtr		fix;
 	BoxVectorPtr	resize;
     } u;
+    int dims[0];
 } Array;
 
-#define ArrayDims(a)	    ((int *) ((a) + 1))
-#define ArrayLimits(a)	    (ArrayDims(a) + (a)->ndim)
-#define ArrayConstant
-#define ArrayNvalues(a)	    ((a)->resizable ? (a)->u.resize->nvalues : (a)->u.fix->nvalues)
-#define ArrayValueBox(a,i)  ((a)->resizable ? BoxVectorBoxes((a)->u.resize)[i] : (a)->u.fix)
-#define ArrayValueElt(a,i)  ((a)->resizable ? 0 : (i))
-#define ArrayType(a)	    ((a)->resizable ? (a)->u.resize->type : (a)->u.fix->u.type)
-#define ArrayValue(a,i)	    (BoxValue(ArrayValueBox(a,i),ArrayValueElt(a,i)))
-#define ArrayValueGet(a,i)  (BoxValueGet(ArrayValueBox(a,i),ArrayValueElt(a,i)))
-#define ArrayValueSet(a,i,v) (BoxValueSet(ArrayValueBox(a,i),ArrayValueElt(a,i), v))
-
 typedef struct _io_chain {
     struct _io_chain	*next;
     int			size;
     int			used;
     int			ptr;
+    unsigned char	buffer[0];
 } FileChain, *FileChainPtr;
 
 typedef struct _file {
@@ -652,7 +670,7 @@ typedef struct _file {
 #define FileEOF		-1
 #define FileBlocked	-2
 #define FileError	-3
-#define FileBuffer(ic)	((unsigned char *) ((ic) + 1))
+#define FileBuffer(ic)	((ic)->buffer)
 
 #define FileReadable	    0x0001
 #define FileWritable	    0x0002
@@ -673,10 +691,20 @@ typedef struct _boxTypes {
     DataType	*data;
     int		count;
     int		size;
+    TypePtr	elements[0];
 } BoxTypes, *BoxTypesPtr;
 
-#define BoxTypesElements(bt)	((TypePtr *) ((bt) + 1))
-#define BoxTypesValue(bt,e)	(BoxTypesElements(bt)[e])
+static inline TypePtr *BoxTypesElements(BoxTypes *bt) {
+	return bt->elements;
+}
+
+static inline TypePtr BoxTypesValue(BoxTypes *bt, int e) {
+	return BoxTypesElements(bt)[e];
+}
+
+static inline void BoxTypesValueSet(BoxTypes *bt, int e, TypePtr t) {
+	BoxTypesElements(bt)[e] = t;
+}
 
 extern BoxTypesPtr  NewBoxTypes (int size);
 extern int	    AddBoxType (BoxTypesPtr *btp, TypePtr t);
@@ -687,22 +715,14 @@ typedef struct _ref {
     int		element;
 } Ref;
 
-int  RefRewrite (Value r);
-
-#define RefCheck(r)	    ((r)->ref.box->replace ? RefRewrite(r) : 0)
-#define RefValueSet(r,v)    (RefCheck (r), BoxValueSet((r)->ref.box, (r)->ref.element, (v)))
-#define RefValue(r)	    (RefCheck (r), BoxValue((r)->ref.box, (r)->ref.element))
-#define RefValueGet(r)	    (RefCheck (r), BoxValueGet((r)->ref.box, (r)->ref.element))
-#define RefType(r)	    (RefCheck (r), BoxType((r)->ref.box, (r)->ref.element))
-#define RefConstant(r)	    BoxConstant((r)->ref.box, (r)->ref.element)
-
 typedef struct _structType {
     DataType	*data;
     int		nelements;
     BoxTypesPtr	types;
+    Atom	atoms[0];
 } StructType;
 
-#define StructTypeAtoms(st)	((Atom *) (st + 1))
+#define StructTypeAtoms(st)	((st)->atoms)
 
 typedef struct _struct {
     BaseValue	base;
@@ -715,6 +735,7 @@ typedef struct _union {
     StructType	*type;
     Atom	tag;
     BoxPtr	value;
+    Type	*types[0];
 } Union;
 
 typedef struct _func {
@@ -907,6 +928,24 @@ struct _valueType {
     Hash	hash;
 };
 
+static inline ValueRep *_ValueRep(Value v) {
+	if (ValueIsInt(v))
+		return &IntRep;
+	return v->value.type;
+}
+
+static inline Rep ValueTag(Value v) {
+	return ValueRep(v)->tag;
+}
+
+static inline Natural *IntegerMag(Value i) {
+	return (Natural *) ((long) (i->integer.magn) & ~1);
+}
+
+static inline Sign IntegerSign(Value i) {
+	return (Sign) ((long) (i->integer.magn) & 1);
+}
+
 typedef struct _boxReplace {
     DataType	    *data;
     BoxPtr	    new;
@@ -925,21 +964,49 @@ typedef struct _box {
 	TypePtr		    type;
 	BoxReplacePtr	    replace;
     } u;
+    Value	    values[0];
 } Box;
 
 #if 1
-#define BoxCheck(box)		assert (!(box)->replace),
+#define BoxCheck(box)		assert (!(box)->replace)
 #else
 #define BoxCheck(box)
 #endif
-#define BoxElements(box)	(BoxCheck (box) ((Value *) ((box) + 1)))
-#define BoxValueSet(box,e,v)	((BoxElements(box)[e]) = (v))
-#define BoxValueGet(box,e)	((BoxElements(box)[e]))
-#define BoxConstant(box,e)	((box)->constant)
-#define BoxReplace(box)		((box)->replace)
-#define BoxType(box,e)		(BoxCheck (box) ((box)->homogeneous ? (box)->u.type : \
-				 BoxTypesValue((box)->u.types, e)))
-				 
+
+static inline Value *BoxElements(Box *box)
+{
+	BoxCheck(box);
+	return box->values;
+}
+
+static inline Value BoxValueSet(Box *box, long e, Value v) {
+	return BoxElements(box)[e] = v;
+}
+
+static inline Value BoxValueGet(Box *box, long e) {
+	return BoxElements(box)[e];
+}
+
+static inline Bool BoxConstant(Box *box, int e) {
+	return box->constant;
+}
+
+static inline Bool _BoxReplace(Box *box) {
+	return box->replace;
+}
+
+#define BoxReplace(box) _BoxReplace(box)
+
+static inline TypePtr _BoxType(Box *box, long e) {
+	BoxCheck(box);
+	if (box->homogeneous)
+		return box->u.type;
+	else
+		return BoxTypesValue(box->u.types, e);
+}
+
+#define BoxType(box, e) _BoxType(box, e)
+
 extern BoxPtr	NewBox (Bool constant, Bool array, int nvalues, TypePtr type);
 extern BoxPtr	NewTypedBox (Bool array, BoxTypesPtr types);
 void		BoxSetReplace (BoxPtr old, BoxPtr new, int oldstride, int newstride);
@@ -948,11 +1015,85 @@ BoxPtr		BoxRewrite (BoxPtr box, int *ep);
 typedef struct {
     DataType	*data;
     int		size;
+    char	values[0];
 } DataCache, *DataCachePtr;
 
 DataCachePtr	NewDataCache (int size);
 
-#define DataCacheValues(vc)	((void *) ((vc) + 1))
+static inline void *DataCacheValues(DataCache *vc) {
+	return (void *) vc->values;
+}
+
+static inline int *ArrayDims (Array *a)
+{
+	return a->dims;
+}
+
+#define ArrayLimits(a)	    (ArrayDims(a) + (a)->ndim)
+#define ArrayConstant
+
+static inline long ArrayNvalues(Array *a) {
+	if (a->resizable)
+		return a->u.resize->nvalues;
+	else
+		return a->u.fix->nvalues;
+}
+
+static inline BoxPtr ArrayValueBox(Array *a, long i) {
+	if (a->resizable)
+		return BoxVectorBoxes(a->u.resize)[i];
+	else
+		return a->u.fix;
+}
+
+static inline int ArrayValueElt(Array *a, long i) {
+	if (a->resizable)
+		return 0;
+	else
+		return i;
+}
+
+static inline TypePtr ArrayType(Array *a) {
+	if (a->resizable)
+		return a->u.resize->type;
+	else
+		return a->u.fix->u.type;
+}
+
+#define ArrayValue(a,i)	    (BoxValue(ArrayValueBox(a,i),ArrayValueElt(a,i)))
+#define ArrayValueGet(a,i)  (BoxValueGet(ArrayValueBox(a,i),ArrayValueElt(a,i)))
+
+static inline void ArrayValueSet(Array *a, long i, Value v)
+{
+	BoxValueSet(ArrayValueBox(a,i),ArrayValueElt(a,i), v);
+}
+
+void
+RefRewrite (Value r);
+
+static inline void RefCheck(Value r) {
+	if (BoxReplace(r->ref.box))
+		RefRewrite(r);
+}
+
+static inline void RefValueSet(Value r, Value v) {
+	RefCheck(r);
+	BoxValueSet(r->ref.box, r->ref.element, v);
+}
+
+static inline Value RefValueGet(Value r) {
+	RefCheck(r);
+	return BoxValueGet(r->ref.box, r->ref.element);
+}
+
+static inline TypePtr RefType (Value r) {
+	RefCheck(r);
+	return BoxType(r->ref.box, r->ref.element);
+}
+
+static inline Bool RefConstant(Value r) {
+	return BoxConstant(r->ref.box, r->ref.element);
+}
 
 Value	NewInteger (Sign sign, Natural *mag);
 Value	NewIntInteger (int value);
@@ -1002,8 +1143,6 @@ int	StringCharSize (unsigned c);
 unsigned    StringGet (char *src, long len, int i);
 char	*StrzPart (Value, char *error);
 
-#ifdef HAVE_C_INLINE
-
 static inline Value
 NewRef (BoxPtr box, int element)
 {
@@ -1018,9 +1157,7 @@ NewRef (BoxPtr box, int element)
     }
     return NewRefReal (box, element, re);
 }
-#else
-Value	NewRef (BoxPtr box, int element);
-#endif
+
 Value	NewStruct (StructType *type, Bool constant);
 StructType  *NewStructType (int nelements);
 Type	*BuildStructType (int nelements, ...);
@@ -1130,7 +1267,7 @@ Bool	Print (Value, Value, char format, int base, int width, int prec, int fill);
 void	PrintError (char *s, ...);
 HashValue HashCrc32 (unsigned char *bytes, int nbytes);
 Value	CopyMutable (Value v);
-#ifdef HAVE_C_INLINE
+
 static inline Value
 Copy (Value v)
 {
@@ -1138,9 +1275,7 @@ Copy (Value v)
 	return CopyMutable (v);
     return v;
 }
-#else
-Value	Copy (Value);
-#endif
+
 Value	ValueEqual (Value a, Value b, int expandOk);
 
 Value	ValueHash (Value a);
@@ -1201,22 +1336,10 @@ int	IntPart (Value, char *error);
 
 double	DoublePart (Value av, char *error);
     
-#ifndef Numericp
-Bool	Numericp (Rep);
-#endif
-#ifndef Integralp
-Bool	Integralp (Rep);
-#endif
 Bool	Zerop (Value);
 Bool	Negativep (Value);
 Bool	Evenp (Value);
 
-#ifdef HAVE_C_INLINE
-#define INLINE inline
-#else
-#define INLINE
-#endif
-
 int	ArrayInit (void);
 int	AtomInit (void);
 int	FileInit (void);
@@ -1232,8 +1355,13 @@ int	RefInit (void);
 int	ForeignInit (void);
 int	ValueInit (void);
 
-# define oneNp(n)	((n)->length == 1 && NaturalDigits(n)[0] == 1)
-# define zeroNp(n)	((n)->length == 0)
+static inline Bool oneNp (Natural *n) {
+	return n->length == 1 && NaturalDigits(n)[0] == 1;
+}
+
+static inline Bool zeroNp (Natural *n) {
+	return n->length == 0;
+}
 
 void	ferr(int);
 void	ignore_ferr (void);
